/*
 *      config.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "config.h"

GKeyFile *initConfig(){
  GKeyFile* ConfigLda = NULL;
  gchar *fileUserConfig = NULL;
  gchar *fileXML = NULL;
  gchar *dirUserConfig = NULL;
  FILE *fic = NULL;
  
  ConfigLda = g_key_file_new();
  
  fileUserConfig = g_strdup_printf("%s%s%s", getenv("HOME"), DIR_USER_CONFIG, FILE_USER_CONFIG);
  
  if(g_file_test(fileUserConfig, G_FILE_TEST_EXISTS) == FALSE){
    dirUserConfig = g_strdup_printf("%s%s", getenv("HOME"), DIR_USER_CONFIG);
    if(g_file_test(dirUserConfig, G_FILE_TEST_IS_DIR) == FALSE){
      if(g_mkdir(dirUserConfig, 0750) == -1){
	fprintf(stderr, "Fail to create preference's directory\n");
	g_free(dirUserConfig);
	g_free(fileUserConfig);
	return NULL;
      }
    }
    /* Génération du fichier de config */
    fic = fopen(fileUserConfig, "w");
    if(fic == NULL){
      fprintf(stderr, "Fail to create the config's file\n");
      g_free(dirUserConfig);
      g_free(fileUserConfig);
      return NULL;
    }
    
    fprintf(fic, "\
[general]\n\
lang=fr\n\
devise=euros\n\
\n\
[view]\n\
window_width=950\n\
window_height=600\n\
sort_category=-1;-1\n\
sort_product=-1;-1\n\
sort_product_added=-1;-1\n\
\n\
[file]\n\
file_list_product=%s/list.xml\n\
last_list_opened=\n\
\n\
[product]\n\
show_column_category_in_product_added=true\n\
show_column_subcategory_in_product_added=true\n\
show_column_price=true\n\
show_column_quantity=true\n\
show_total_price=true\n\
show_total_quantity=true\n\
\n\
[print]\n\
police=Monospace\n\
size_police=10.5\n\
show_title=true\n\
show_page=true\n\
color=0;0;0\n\
border=1\n\
show_column_number=true\n\
show_column_name=true\n\
show_column_price=true\n\
show_column_quantity=true\n\
show_line_total=true\n\
length_maxi_number=3\n\
length_maxi_name=20\n\
length_maxi_price=10\n\
length_maxi_quantity=10\n\
cellpadding=3\n", dirUserConfig);
    fclose(fic);
    
    fileXML = g_strdup_printf("%s/list.xml", dirUserConfig);
    
    fic = fopen(fileXML, "w");
    if(fic == NULL){
      fprintf(stderr, "Fail to create the file with list of product\n");
      g_free(fileXML);
      g_free(dirUserConfig);
      g_free(fileUserConfig);
      return NULL;
    }
    fprintf(fic, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<lda/>\n");
    fclose(fic);
    
    g_free(fileXML);
    g_free(dirUserConfig);
  }
  
  if(g_key_file_load_from_file(ConfigLda, (const gchar*)fileUserConfig, G_KEY_FILE_NONE, NULL) == FALSE){
    fprintf(stderr, "Fail to load the config...\n");
    g_free(fileUserConfig);
    return NULL;
  }
  
  checkCompatibilityConfig(ConfigLda);
  addedNewOption(ConfigLda);
  
  saveConfig(ConfigLda, fileUserConfig);
  
  g_free(fileUserConfig);
  return ConfigLda;
}

gboolean checkCompatibilityConfig(GKeyFile* ConfigLda){
	gchar *pText = NULL;
	
	pText = g_key_file_get_string(ConfigLda, "file", "listeFile", NULL);
	if(pText){
		/* remove the key "listeFile" */
		g_key_file_remove_key(ConfigLda, "file", "listeFile", NULL);
		/* set new key "file_list_product" */
		g_key_file_set_string(ConfigLda, "file", "file_list_product", pText);
		g_free(pText);
		
		/* Add new options */
		g_key_file_set_boolean(ConfigLda, "product", "show_column_price", TRUE);
		g_key_file_set_boolean(ConfigLda, "product", "show_column_quantity", TRUE);
		g_key_file_set_boolean(ConfigLda, "product", "show_total_price", TRUE);
		g_key_file_set_boolean(ConfigLda, "product", "show_total_quantity", TRUE);
		
		return FALSE;
	}
	return TRUE;
}

void addedNewOption(GKeyFile* ConfigLda){
	gboolean valueBoolean;
	gint valueInt;
	gint *valueIntList;
	gsize length;
	GError *error = NULL;
	
	valueBoolean = g_key_file_get_boolean(ConfigLda, "product", "show_column_category_in_product_added", &error);
	if(error){
		g_key_file_set_boolean(ConfigLda, "product", "show_column_category_in_product_added", TRUE);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueBoolean = g_key_file_get_boolean(ConfigLda, "product", "show_column_subcategory_in_product_added", &error);
	if(error){
		g_key_file_set_boolean(ConfigLda, "product", "show_column_subcategory_in_product_added", TRUE);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueInt = g_key_file_get_integer(ConfigLda, "view", "window_width", &error);
	if(error){
		g_key_file_set_integer(ConfigLda, "view", "window_width", 950);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueInt = g_key_file_get_integer(ConfigLda, "view", "window_height", &error);
	if(error){
		g_key_file_set_integer(ConfigLda, "view", "window_height", 600);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueIntList = g_key_file_get_integer_list(ConfigLda, "view", "sort_category", &length, &error);
	if(error){
		gint sort_category[2];
		sort_category[0] = -1;
		sort_category[1] = -1;
		
		g_key_file_set_integer_list(ConfigLda, "view", "sort_category", sort_category, (gsize)2);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueIntList = g_key_file_get_integer_list(ConfigLda, "view", "sort_product", &length, &error);
	if(error){
		gint sort_product[2];
		sort_product[0] = -1;
		sort_product[1] = -1;
		
		g_key_file_set_integer_list(ConfigLda, "view", "sort_product", sort_product, (gsize)2);
		
		g_error_free(error);
		error = NULL;
	}
	
	valueIntList = g_key_file_get_integer_list(ConfigLda, "view", "sort_product_added", &length, &error);
	if(error){
		gint sort_product_added[2];
		sort_product_added[0] = -1;
		sort_product_added[1] = -1;
		
		g_key_file_set_integer_list(ConfigLda, "view", "sort_product_added", sort_product_added, (gsize)2);
		
		g_error_free(error);
		error = NULL;
	}
}

gboolean saveConfig(GKeyFile* ConfigLda, gchar* filename){
	FILE *file = NULL;
	gchar *pConfigData;
	
	pConfigData = g_key_file_to_data(ConfigLda, NULL, NULL);
  file = fopen(filename, "w");
  if(!file){
  	fprintf(stderr, "Fail to open the config's file\n");
  	return FALSE;
  }
  fprintf(file, "%s", pConfigData);
  g_free(pConfigData);
  fclose(file);
  return TRUE;
}

void freeConfig(GKeyFile* ConfigLda){
  g_key_file_free(ConfigLda);
}
